#include "AMDiS.hpp"

// std c++ headers
#include <string>

#include <boost/program_options.hpp>

// AMDiS includes
#include <dune/amdis/Initfile.hpp>
#include <dune/amdis/Output.hpp>

namespace AMDiS
{
//   using namespace std;

  Dune::MPIHelper& init(int& argc, char**& argv, std::string initFileName)
  {
    // Maybe initialize MPI
    Dune::MPIHelper& mpiHelper = Dune::MPIHelper::instance(argc, argv);

    Parameters::clearData();

    // read commandline arguments
    namespace po = boost::program_options;

    // Declare the supported options.
    po::options_description desc("Usage: " + std::string(argv[0]) + " init-file [options]\nAllowed options");
    desc.add_options()
    ("help", "produce help message")
    ("init-file", po::value<std::string>(), "set init file")
    ("parameters", po::value<std::string>(), "set parameter in init file\nsyntax: \"key1: value1; key2: value2...\"");

    po::options_description hidden("Hidden options");
    hidden.add_options()
    ("unknown", po::value<std::vector<std::string>>(), "unknown options");

    po::options_description cmdline_options;
    cmdline_options.add(desc).add(hidden);

    // first argument is init-filename
    po::positional_options_description p;
    p.add("init-file", 1);
    p.add("unknown", -1);

    // parse comandline
    po::variables_map vm;
    po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).allow_unregistered().run(), vm);
    po::notify(vm);

    // print help message
    if (vm.count("help"))
    {
      std::cout << desc << "\n";
      exit(1);
    }

    // set parameters before reading the initfile
    // if (vm.count("parameters"))
    //   Parameters::readArgv(vm["parameters"].as<std::string>());

    if (initFileName == "")
    {
      if (vm.count("init-file"))
        Parameters::init(vm["init-file"].as<std::string>());
      else
        error_exit("No init file specified!\n");
    }
    else
    {
      Parameters::init(initFileName);
    }

    // reset parameters from command line
    bool ignoreCommandline = false;
    Parameters::get("ignore commandline options", ignoreCommandline);
    // if (vm.count("parameters") && !ignoreCommandline)
    //   Parameters::readArgv(vm["parameters"].as<std::string>(),0);
    // TODO: add command-line arguments again.

    return mpiHelper;
  }


  void init(std::string initFileName)
  {
    Parameters::init(initFileName);
  }


  void finalize()
  {}

} // end namespace AMDiS
