/** \file Log.hpp */

#pragma once

// std c++ headers: std::ostream, std::ofstream
#include <iostream>
#include <sstream>

#ifdef MSG 
#undef MSG 
#endif
#ifdef ERROR 
#undef ERROR 
#endif
#ifdef WARNING 
#undef WARNING 
#endif
#ifdef TEST 
#undef TEST 
#endif
#ifdef TEST_EXIT 
#undef TEST_EXIT 
#endif


#define AMDIS_STATIC_ASSERT(...) \
  static_assert(__VA_ARGS__, #__VA_ARGS__)

// ===========================================================================
// ===== message macros ======================================================
// ===========================================================================

#define AMDIS_UNUSED(var) __attribute__((unused)) var

/// Should be the first call in every functions. It defines the current
/// function name nn for message output via MSG, WARNING, ...
#define AMDIS_FUNCNAME(nn) AMDIS_UNUSED(const char *funcName); funcName = nn;

#ifdef NDEBUG
  #define AMDIS_FUNCNAME_DBG(nn)
#else
  #define AMDIS_FUNCNAME_DBG(nn) AMDIS_UNUSED(const char *funcName); funcName = nn;
#endif


  /// prints a message
#define AMDIS_MSG(text) std::cout << text << "\n"

  /// prints an error message
#define AMDIS_ERROR(text) AMDIS_MSG("Error: " << text)

  /// prints an error message and exits
#define AMDIS_ERROR_EXIT(text) do { std::stringstream ss; ss << "Error: " << text; throw std::runtime_error(ss.str()); } while(0)

  /// prints a warning
#define AMDIS_WARNING(text) AMDIS_MSG("Warning: " << text)

  /// if test is false, an error message is printed
#define AMDIS_TEST(test, msg) if (!(test)) { AMDIS_WARNING(msg); }

  /// if test is false, an error message is printed and the program exits
#define AMDIS_TEST_EXIT(test, msg) if (!(test)) { AMDIS_ERROR_EXIT(msg); }

  /// In debug mode, it corresponds to ERROR_EXIT, otherwise it is noop.
#ifdef NDEBUG
  #define AMDIS_TEST_EXIT_DBG(test, msg)
  #define AMDIS_TEST_DBG(test, msg)
  #define AMDIS_MSG_DBG(text)
  #define AMDIS_DBG_VAR(var)
#else
  #define AMDIS_TEST_EXIT_DBG(test, msg) AMDIS_TEST_EXIT(test, msg)
  #define AMDIS_TEST_DBG(test, msg) AMDIS_TEST(test, msg)
  #define AMDIS_MSG_DBG(text) AMDIS_MSG(text)
  #define AMDIS_DBG_VAR(var) var
#endif
