#include "ProblemInstat.hpp"

#include <dune/amdis/AdaptInfo.hpp>

namespace AMDiS
{
  template <class Traits>
  void ProblemInstat<Traits>::transferInitialSolution(AdaptInfo& adaptInfo)
  {
    AMDIS_FUNCNAME("ProblemInstat::transferInitialSolution()");

    test_exit(adaptInfo.getTime() == adaptInfo.getStartTime(),
              "after initial solution: time != start time");
    problemStat.writeFiles(adaptInfo, true);
  }


  template <class Traits>
  void ProblemInstat<Traits>::closeTimestep(AdaptInfo& adaptInfo)
  {
    bool force = (adaptInfo.getTime() >= adaptInfo.getEndTime());
    problemStat.writeFiles(adaptInfo, force);
  }


  template <class Traits>
  void ProblemInstat<Traits>::initialize(Flag initFlag)
  {
    // === create vector for old solution ===
    if (initFlag.isSet(INIT_UH_OLD))
        createUhOld();
  }


  template <class Traits>
  void ProblemInstat<Traits>::createUhOld()
  {
    AMDIS_FUNCNAME("ProblemInstat::createUhOld()");

    if (oldSolution) {
      warning("oldSolution already created\n");
    }
    else {
      const int size = problemStat.getNumComponents();

      // create oldSolution
      std::vector<std::string> componentNames(size, name + "_uOld");
      oldSolution = std::make_shared<SystemVectorType>(*problemStat.getFeSpaces(), componentNames);
    }
  }


  template <class Traits>
  void ProblemInstat<Traits>::initTimestep(AdaptInfo&)
  {
    if (oldSolution)
      oldSolution->copy(*problemStat.getSolution());
  }

}
