#pragma once

#include "ProblemStatBase.hpp"
#include "ProblemTimeInterface.hpp"

namespace AMDiS
{
  // forward declarations
  class AdaptInfo;

  /**
   * \ingroup Problem
   *
   * \brief
   * Base class for \ref ProblemInstat.
   */
  class ProblemInstatBase
      : public ProblemTimeInterface
      , public ProblemStatBase   // NOTE: Why is this derived from ProblemStatBase
  {
  public:
    /// Constructor.
    ProblemInstatBase(std::string probName,
                      ProblemStatBase* initialProb)
      : name(probName)
      , initialProblem(initialProb ? initialProb : this)
    {}

    /// Destructor.
    virtual ~ProblemInstatBase() {}

    /// Implementation of \ref ProblemTimeInterface::setTime().
    virtual void setTime(AdaptInfo& adaptInfo) override;

    void solve(AdaptInfo&) { /* by default, do nothing */ }

    /// Implementation of \ref ProblemStatBase::solve().
    virtual void solve(AdaptInfo& adaptInfo, bool, bool) override
    {
      solve(adaptInfo);
    }

    /// Implementation of \ref ProblemStatBase::estimate().
    virtual void estimate(AdaptInfo&) override { /* by default, do nothing */ }

    /// Implementation of \ref ProblemStatBase::buildBeforeRefine().
    virtual void buildBeforeRefine(AdaptInfo&, Flag) override { /* by default, do nothing */ }

    /// Implementation of \ref ProblemStatBase::buildBeforeCoarsen().
    virtual void buildBeforeCoarsen(AdaptInfo&, Flag) override { /* by default, do nothing */ }

    /// Implementation of \ref ProblemStatBase::buildAfterCoarsen().
    virtual void buildAfterCoarsen(AdaptInfo&, Flag, bool, bool) override { /* by default, do nothing */ }

    /// Implementation of \ref ProblemStatBase::markElements().
    virtual Flag markElements(AdaptInfo&) override
    {
      return {0};
    }

    /// Implementation of \ref ProblemStatBase::refineMesh().
    virtual Flag refineMesh(AdaptInfo&) override
    {
      return {0};
    }

    /// Implementation of \ref ProblemStatBase::coarsenMesh().
    virtual Flag coarsenMesh(AdaptInfo&) override
    {
      return {0};
    }

    /// Implementation of \ref ProblemTimeInterface::closeTimestep().
    virtual void closeTimestep(AdaptInfo&) override { /* by default, do nothing */ }

    /// Implementation of \ref ProblemStatBase::getName().
    virtual std::string getName() const override
    {
      return name;
    }

    /// Implementation of \ref ProblemTimeInterface::solveInitialProblem().
    virtual void solveInitialProblem(AdaptInfo& adaptInfo) override;

    double* getTime()
    {
      return &cTime;
    }

    double* getTau()
    {
      return &tau;
    }

    double* getInvTau()
    {
      return &invTau;
    }

  protected:
    /// Name of the problem.
    std::string name;

    ProblemStatBase* initialProblem;

    /// Time
    double cTime = 0.0;

    /// Timestep
    double tau = 1.0;

    /// 1 / timestep
    double invTau = 1.0;
  };

} // end namespace AMDiS
