#pragma once

// from http://stackoverflow.com/questions/17424477/implementation-c14-make-integer-sequence/17426611#17426611
namespace AMDiS
{
  template <size_t I>
  using index_ = std::integral_constant<size_t, I>;

  /// class that represents a sequence of indices
  template <size_t...>
  struct Indices {};

  /// Some predefined index constants
  static constexpr index_<0> _0  {};
  static constexpr index_<1> _1  {};
  static constexpr index_<2> _2  {};
  static constexpr index_<3> _3  {};
  static constexpr index_<3> _4  {};
  static constexpr index_<3> _5  {};
  static constexpr index_<3> _6  {};
  static constexpr index_<3> _7  {};
  static constexpr index_<3> _8  {};
  static constexpr index_<3> _9  {};


  template <size_t i, size_t j>
  constexpr index_<i+j> operator+(index_<i>, index_<j>) { return {}; }

  template <size_t i, size_t j>
  constexpr index_<i-j> operator-(index_<i>, index_<j>) { return {}; }

  template <size_t i, size_t j>
  constexpr index_<i*j> operator*(index_<i>, index_<j>) { return {}; }

  template <size_t i, size_t j>
  constexpr index_<i/j> operator/(index_<i>, index_<j>) { return {}; }


  namespace Impl
  {
    template <size_t s, class S>
    struct ConcatSeq;

    template <size_t s, size_t ... i>
    struct ConcatSeq<s, Indices<i... >>
    {
      using type = Indices<i..., (s + i)... >;
    };

    template <bool, class S>
    struct IncSeq_if
    {
      using type = S;
    };

    template <size_t... Is>
    struct IncSeq_if<true, Indices<Is...>>
    {
      using type = Indices<Is..., sizeof...(Is)>;
    };

  } // end namespace Impl

  template <size_t N>
  struct MakeSeq
  {
    using type = typename Impl::IncSeq_if< (N % 2 != 0),
      typename Impl::ConcatSeq<N/2, typename MakeSeq<N/2>::type>::type >::type;
  };

  // break condition
  template <> struct MakeSeq<0> { using type = Indices<>; };

  /// Alias template to create a sequence of indices
  template <size_t N>
  using MakeSeq_t = typename MakeSeq<N>::type;

} // end namespace AMDiS
