#pragma once

#include <dune/amdis/common/Utility.hpp>

namespace AMDiS
{  
  /// class that performes the loop over all indices [I, N), for i < N
  template <size_t I, size_t N>
  struct For
  {
    template <class F>
    static void loop(F&& f) 
    {
      f(index_<I>());        		     // call the functor on the ith index
      For<I+1, N>::loop(std::forward<F>(f)); // go to next loop iteration
    }
  };

  // default behavior (i==N): do nothing
  template <size_t N>
  struct For<N, N> { template <class F> static void loop(F) {} };

  /// Function to call static loop \ref For
  template <size_t I, size_t N, class F>
  std::enable_if_t< (I <= N) > for_(F&& f)
  {
    For<I, N>::loop(std::forward<F>(f));
  }
  
} // end namespace AMDiS
