#pragma once

namespace AMDiS
{
  namespace Impl 
  {
    // Type-Trait needs to be specialized by linear algebra libraries
    template <class Vector>
    struct BaseVector { using type = Vector; };
    
    template <class Matrix>
    struct BaseMatrix { using type = Matrix; };
  }

  template <class Vector>
  using BaseVector_t = typename Impl::BaseVector<Vector>::type;

  template <class Matrix>
  using BaseMatrix_t = typename Impl::BaseMatrix<Matrix>::type;
    
  
  namespace Impl
  {
    template <class Vector>
    class BaseWrapper
    {
    public:
      using value_type = typename Vector::value_type;
      using size_type = typename Vector::size_type;
      
      BaseWrapper(Vector& vec) : vec(vec) {}
      
      void resize(size_type s)
      {
        vec.resize(s);
      }
      
      size_type size() const
      {
        return vec.size();
      }
    
      value_type&       operator[](size_type i)       { return vec[i]; }
      value_type const& operator[](size_type i) const { return vec[i]; }
      
    protected:
      Vector& vec;
    };
    
  } // end namespace Impl
  
  template <class Vector>
  Impl::BaseWrapper<std::remove_reference_t<Vector>> wrapper(Vector&& vec)
  {
    return {std::forward<Vector>(vec)};
  }
  
} // end namespace AMDiS
