/** \file ITL_Preconditioner.h */

#pragma once

#include <dune/amdis/linear_algebra/PreconditionerInterface.hpp>

namespace AMDiS
{
  /**
   * \ingroup Solver
   *
   * \brief Wrapper for using ITL preconditioners in AMDiS.
   */
  template <class Matrix, class Vector, class PreconRunner>
  class Preconditioner : public PreconditionerInterface<Matrix, Vector>
  {
    using Self = Preconditioner;
    using Super = PreconditionerInterface<Matrix, Vector>;

  public:
    /// A creator to be used instead of the constructor.
    struct Creator : CreatorInterface<Super>
    {
      virtual shared_ptr<Super> create() override
      {
        return make_shared<Self>();
      }
    };

  public:
    /// Implementation of \ref PreconditionerBase::init()
    virtual void init(Matrix const& fullMatrix) override
    {
      precon.reset(new PreconRunner(fullMatrix));
    }

    /// Implementation of \ref PreconditionerInterface::exit()
    virtual void exit() override
    {
      precon.reset();
    }

    /// Implementation of \ref PreconditionerBase::solve()
    virtual void solve(Vector const& vin, Vector& vout) const override
    {
      test_exit_dbg(bool(precon), "No preconditioner initialized!");
      precon->solve(vin, vout);
    }

    /// Implementation of \ref PreconditionerBase::adjoint_solve()
    virtual void adjoint_solve(Vector const& vin, Vector& vout) const override
    {
      test_exit_dbg(bool(precon), "No preconditioner initialized!");
      precon->adjoint_solve(vin, vout);
    }

  private:
    shared_ptr<PreconRunner> precon;
  };

} // namespace AMDiS
