#pragma once

#include <type_traits>

#include <dune/amdis/OperatorTermBase.hpp>
#include <dune/amdis/common/ScalarTypes.hpp>

namespace AMDiS
{
  /**
    * \defgroup OperatorTerms Expressions and terms to be used in GenericOperatorTerm.
    * @{
    **/

  /// An expression representing a constant (arithmetic) value, \see constant
  template <class ValueType>
  class ConstantTerm
  {
  public:
    // TODO: possibly convert to plain type, in case of expression templates.
    using value_type = ValueType;

    ConstantTerm(value_type value)
      : value(value)
    {}

    template <class Element, class PointList>
    void init(Element const& element,
              PointList const& points) { /* do nothing */ }

    value_type const& operator[](size_t iq) const
    {
      return value;
    }

    int getDegree(Dune::GeometryType const&) const
    {
      return 0;
    }

  private:
    value_type value;
  };


  /// generator function for \ref ConstantTerm expressions
  template <class T>
  ConstantTerm<T> constant(T value) { return {value}; }

  /** @} **/

} // end namespace AMDiS
