#pragma once

#include <dune/amdis/AMDiS.hpp>
#include <dune/amdis/LinearAlgebra.hpp>

namespace AMDiS
{
  /// Basis preconditioner structure for block-preconditioners
  template <class Matrix, class Vector>
  class BlockPreconditioner : public PreconditionerInterface<Matrix, Vector>
  {
  public:
    using Self = BlockPreconditioner;
    using Super = PreconditionerInterface<Matrix, Vector>;
    using PreconBase = Super;

  public:
    BlockPreconditioner() = default;

    /// extract iranges from BlockMatrix to be used to extract sub-vectors and sub-matrices.
    virtual void init(Matrix const& A_) override
    {
      A = &A_;

      A_.getRanges(rows, cols);
      initialized = true;
    }

    virtual void solve(Vector const& b, Vector& x) const override
    {
      error_exit("Must be implemented in derived classes!\n");
    }

    virtual void adjoint_solve(Vector const& x, Vector& y) const override
    {
      error_exit("Must be implemented in derived classes!\n");
    }

    template <size_t I>
    mtl::irange const& getRowRange(const index_<I> i = {}) const
    {
      test_exit_dbg(initialized,
                    "BlockPreconditioner not initialized. Call init() before.");
      return std::get<I>(rows);
    }

    template <size_t I>
    mtl::irange const& getColRange(const index_<I> i = {}) const
    {
      test_exit_dbg(initialized,
                    "BlockPreconditioner not initialized. Call init() before.");
      return std::get<I>(cols);
    }

  protected:
    Matrix const* A = NULL;
    bool initialized = false;

    std::array<mtl::irange, Matrix::N()> rows;
    std::array<mtl::irange, Matrix::M()> cols;
  };

} // end namespace AMDiS
