// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:

#include <iostream>

#include <dune/amdis/AMDiS.hpp>
#include <dune/amdis/ProblemStat.hpp>
#include <dune/amdis/Terms.hpp>

using namespace AMDiS;

// 1 component with polynomial degree 1
using HeatParam   = ProblemStatTraits<AMDIS_DIM, AMDIS_DOW, 2>;
using HeatProblem = ProblemStat<HeatParam>;

int main(int argc, char** argv)
{
    AMDiS::init(argc, argv);
    
    HeatProblem prob("heat");
    prob.initialize(INIT_ALL);
    
    AdaptInfo adaptInfo("adapt");
    
    double tau = adaptInfo.getTimestep();
    AMDIS_MSG("timestep = " << tau);
    
    using Op = HeatProblem::OperatorType;
    Op opLhs; 
    opLhs.addZOT( constant(1.0/tau) );
    opLhs.addSOT( constant(1.0) );
    
    Op opRhs;  
    opRhs.addZOT( valueOf(prob.getSolution<0>(), 1.0/tau) );
    opRhs.addZOT( eval([](auto const& x) { return -1.0; }) );
    
    prob.addMatrixOperator(opLhs, 0, 0);
    prob.addVectorOperator(opRhs, 0);
    
    // set boundary condition
    auto predicate = [](auto const& p){ return p[0] < 1.e-8 || p[1] < 1.e-8; };
    auto dbcValues = [](auto const& p){ return 0.0; };
    prob.addDirichletBC(predicate, 0, 0, dbcValues);
        
    prob.getSolution<0>().getVector() = 0.0;
    double t = 0.0;
    for (t = adaptInfo.getStartTime();
	 t < adaptInfo.getEndTime();
	 t+= adaptInfo.getTimestep()) 
    {
	prob.writeFiles(adaptInfo, t);
	prob.buildAfterCoarsen(adaptInfo, Flag(0));
	prob.solve(adaptInfo);
    }
    
    prob.writeFiles(adaptInfo, t);
    AMDiS::finalize();
    return 0;
}